/*
 * Copyright (C) 2010 Wayne Meissner
 *
 * This file is part of jnr.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package jnr.netdb;

import java.io.Closeable;
import java.io.IOException;
import java.io.Reader;
import java.util.Iterator;

/**
 *
 */
class NetDBParser implements Iterable<NetDBEntry>, Closeable {
    private final Reader reader;

    public NetDBParser(Reader r) {
        this.reader = r;
    }

    public Iterator<NetDBEntry> iterator() {
        return new NetDBIterator(reader);
    }
    
    public void close() throws IOException {
        reader.close();
    }
}
